@@ -59,7 +59,7 @@ sig
   (** 
     * Returns the unique serial number for the logical bitstream of this page. 
     * Each page contains the serial number for the logical bitstream that it belongs to.*)
-  val serialno : t -> int
+  val serialno : t -> nativeint
 
   (** 
     * Indicates whether this page is at the end of the logical bitstream. *)
@@ -104,13 +104,13 @@ sig
     *
     * For example, in audio codecs this position is the pcm sample number and 
     * in video this is the frame number.*)
-  val granulepos : t -> int
+  val granulepos : t -> Int64.t
 
   (**
     * Returns the sequential page number. 
     * 
     * This is useful for ordering pages or determining when pages have been lost. *)
-  val pageno : t -> int
+  val pageno : t -> nativeint
 
   (**
     * Checksums an ogg_page. *)
@@ -132,7 +132,7 @@ sig
 
   (**
     * Wrapper around [create] to open a file as the ogg stream. *)
-  val create_from_file : string -> t
+  val create_from_file : string -> (t*Unix.file_descr)
 
   (** 
     * Read a page from [Sync.t] 
@@ -163,7 +163,15 @@ sig
   (**
     * Create a [stream].
     *)
-  val create : ?serial:int -> unit -> stream
+  val create : ?serial:nativeint -> unit -> stream
+
+  (** 
+    * Get a stream's serial number.
+    *)
+  val serialno : stream -> nativeint
+
+  (** Returns true if the end of stream has been reached. *)
+  val eos : stream -> bool
 
   (**
     * This function forms packets into pages. Internally,
@@ -209,6 +217,23 @@ sig
     * [sync] is a reference to a boolean set to true if the stream had a gap. *)
   val get_packet : ?sync:(bool ref) -> stream -> packet
 
+  (**
+    * This function assembles a data packet for output
+    * to the codec decoding engine without advancing the stream.
+    *
+    * Raises [Not_enough_data] if more data is needed and another page should be submitted.
+    *
+    * Raises [Out_of_sync]  if we are out of sync and there is a gap in the data *)
+  val peek_packet : stream -> packet
+
+  (**
+    * This function submits a packet to the bitstream for page encapsulation. 
+    * After this is called, more packets can be submitted, or pages can be written out.
+    *
+    * This function is provided to ease ogg strea multiplexing, where packet submission
+    * order is important. It should not be used to encoder further data. *)
+  val put_packet : stream -> packet -> unit
+
   (**
     * This function checks for remaining packets inside the stream and forces
     * remaining packets into a page, regardless of the size of the page.
@@ -237,3 +262,22 @@ sig
 
 end
 
+module Skeleton : 
+sig
+
+  (**
+    * Create an initial ogg skeleton packet ('fishead'),
+    * to complete with data packet from the various codecs
+    * in the stream ('fishbone').
+    * See: http://xiph.org/ogg/doc/skeleton.html. *)
+  val fishead :
+    ?presentation_numerator:Int64.t ->
+    ?presentation_denominator:Int64.t ->
+    ?basetime_numerator:Int64.t ->
+    ?basetime_denominator:Int64.t -> ?utc:Int32.t -> unit -> Stream.packet
+
+ (** Create an end-of-stream packet for
+   * an ogg skeleton logical stream *)
+ val eos : unit -> Stream.packet
+
+end
