@@ -83,7 +83,7 @@ CAMLprim value ocaml_ogg_page_serialno(value page)
 {
   CAMLparam1(page);
   ogg_page op;
-  CAMLreturn(Val_int(ogg_page_serialno(page_of_value(page,&op))));
+  CAMLreturn(caml_copy_nativeint(ogg_page_serialno(page_of_value(page,&op))));
 }
 
 CAMLprim value ocaml_ogg_page_eos(value page)
@@ -125,14 +125,14 @@ CAMLprim value ocaml_ogg_page_granulepos(value page)
 {
   CAMLparam1(page);
   ogg_page op;
-  CAMLreturn(Val_int(ogg_page_granulepos(page_of_value(page,&op))));
+  CAMLreturn(caml_copy_int64(ogg_page_granulepos(page_of_value(page,&op))));
 }
 
 CAMLprim value ocaml_ogg_page_pageno(value page)
 {
   CAMLparam1(page);
   ogg_page op;
-  CAMLreturn(Val_int(ogg_page_pageno(page_of_value(page,&op))));
+  CAMLreturn(caml_copy_nativeint(ogg_page_pageno(page_of_value(page,&op))));
 }
 
 CAMLprim value ocaml_ogg_page_checksum_set(value page)
@@ -197,7 +197,7 @@ CAMLprim value ocaml_ogg_sync_read(value callback, value oy)
     if (Int_val(bytes) == 0)
       caml_raise_constant(*caml_named_value("ogg_exn_not_enough_data"));
 
-    char *buffer = ogg_sync_buffer(sync,len);
+    char *buffer = ogg_sync_buffer(sync,Int_val(bytes));
     memcpy(buffer,String_val(s),Int_val(bytes));
     ogg_sync_wrote(sync,Int_val(bytes));
   }
@@ -209,9 +209,8 @@ CAMLprim value ocaml_ogg_sync_read(value callback, value oy)
 
 static void finalize_stream_state(value s)
 {
-  ogg_stream_state *os = Stream_state_val(s);
-  ogg_stream_clear(os);
-  free(os);
+  // This also free the argument
+  ogg_stream_destroy(Stream_state_val(s));
 }
 
 static struct custom_operations stream_state_ops =
@@ -234,6 +233,8 @@ static void finalize_packet(value s)
 static inline ogg_packet *copy_packet(ogg_packet *op)
 {
   ogg_packet *nop = malloc(sizeof(ogg_packet));
+  if (nop == NULL)
+    caml_failwith("malloc");
   nop->packet = malloc(op->bytes);
   memcpy(nop->packet,op->packet,op->bytes);
   nop->bytes = op->bytes;
@@ -255,19 +256,35 @@ static struct custom_operations packet_ops =
   custom_deserialize_default
 };
 
+value value_of_packet(ogg_packet *op)
+{
+  CAMLparam0();
+  CAMLlocal1(packet);
+  packet = caml_alloc_custom(&packet_ops, sizeof(ogg_packet*), 1, 0);
+  Packet_val(packet) = copy_packet(op);
+  CAMLreturn(packet);
+}
+
 CAMLprim value ocaml_ogg_stream_init(value serial)
 {
   CAMLparam0();
   CAMLlocal1(ans);
   ogg_stream_state *os = malloc(sizeof(ogg_stream_state));
 
-  ogg_stream_init(os, Int_val(serial));
+  ogg_stream_init(os, Nativeint_val(serial));
   ans = caml_alloc_custom(&stream_state_ops, sizeof(ogg_stream_state*), 1, 0);
   Stream_state_val(ans) = os;
 
   CAMLreturn(ans);
 }
 
+CAMLprim value ocaml_ogg_stream_eos(value o_stream_state)
+{
+  CAMLparam1(o_stream_state);
+  ogg_stream_state *os = Stream_state_val(o_stream_state);
+  CAMLreturn(Val_bool(ogg_stream_eos(os)));
+}
+
 CAMLprim value ocaml_ogg_stream_pageout(value o_stream_state)
 {
   CAMLparam1(o_stream_state);
@@ -292,10 +309,20 @@ CAMLprim value ocaml_ogg_stream_pagein(value o_stream_state, value page)
   CAMLreturn(Val_unit);
 }
 
+CAMLprim value ocaml_ogg_stream_packetin(value o_stream_state, value packet)
+{
+  CAMLparam2(o_stream_state, packet);
+  ogg_stream_state *os = Stream_state_val(o_stream_state);
+
+  if (ogg_stream_packetin(os, Packet_val(packet)) != 0)
+    caml_raise_constant(*caml_named_value("ogg_exn_bad_data"));
+
+  CAMLreturn(Val_unit);
+}
+
 CAMLprim value ocaml_ogg_stream_packetout(value callback,value o_stream_state)
 {
   CAMLparam2(callback,o_stream_state);
-  CAMLlocal1(packet);
   ogg_stream_state *os = Stream_state_val(o_stream_state);
   ogg_packet op;
   int ret = ogg_stream_packetout(os,&op);
@@ -306,10 +333,20 @@ CAMLprim value ocaml_ogg_stream_packetout(value callback,value o_stream_state)
   if (ret == -1)
     caml_callback(callback,Val_unit);
 
-  packet = caml_alloc_custom(&packet_ops, sizeof(ogg_packet*), 1, 0);
-  Packet_val(packet) = copy_packet(&op);
+  CAMLreturn(value_of_packet(&op));
+}
 
-  CAMLreturn(packet);
+CAMLprim value ocaml_ogg_stream_packetpeek(value o_stream_state)
+{
+  CAMLparam1(o_stream_state);
+  ogg_stream_state *os = Stream_state_val(o_stream_state);
+  ogg_packet op;
+  int ret = ogg_stream_packetpeek(os,&op);
+
+  if (ret == 0)
+    caml_raise_constant(*caml_named_value("ogg_exn_not_enough_data"));
+
+  CAMLreturn(value_of_packet(&op));
 }
 
 CAMLprim value ocaml_ogg_flush_stream(value o_stream_state)
@@ -324,3 +361,95 @@ CAMLprim value ocaml_ogg_flush_stream(value o_stream_state)
   CAMLreturn(value_of_page(&og));
 }
 
+CAMLprim value ocaml_ogg_stream_serialno(value o_stream_state)
+{
+  CAMLparam1(o_stream_state);
+  ogg_stream_state *os = Stream_state_val(o_stream_state);
+
+  CAMLreturn(caml_copy_nativeint((intnat)os->serialno));
+}
+
+/* Ogg skeleton helpers */
+
+/* Values from http://xiph.org/ogg/doc/skeleton.html */
+#define SKELETON_VERSION_MAJOR 3
+#define SKELETON_VERSION_MINOR 0
+#define FISHEAD_IDENTIFIER "fishead\0"
+
+/* Wrappers */
+static void write16le(unsigned char *ptr,ogg_uint16_t v)
+{
+  ptr[0]=v&0xff;
+  ptr[1]=(v>>8)&0xff;
+}
+
+static void write32le(unsigned char *ptr,ogg_uint32_t v)
+{
+  ptr[0]=v&0xff;
+  ptr[1]=(v>>8)&0xff;
+  ptr[2]=(v>>16)&0xff;
+  ptr[3]=(v>>24)&0xff;
+}
+
+static void write64le(unsigned char *ptr,ogg_int64_t v)
+{
+  ogg_uint32_t hi=v>>32;
+  ptr[0]=v&0xff;
+  ptr[1]=(v>>8)&0xff;
+  ptr[2]=(v>>16)&0xff;
+  ptr[3]=(v>>24)&0xff;
+  ptr[4]=hi&0xff;
+  ptr[5]=(hi>>8)&0xff;
+  ptr[6]=(hi>>16)&0xff;
+  ptr[7]=(hi>>24)&0xff;
+}
+
+/* Code from theorautils.c in ffmpeg2theora */
+CAMLprim value ocaml_ogg_skeleton_fishead(value pres_num, value pres_den, value base_num, value base_den, value time)
+{
+  CAMLparam0();
+  CAMLlocal1(packet);
+  ogg_packet op;
+
+  memset (&op, 0, sizeof (op));
+
+  op.packet = malloc (64);
+  if (op.packet == NULL)
+    caml_failwith("malloc");
+
+  memset (op.packet, 0, 64);
+  memcpy (op.packet, FISHEAD_IDENTIFIER, 8); /* identifier */
+  write16le(op.packet+8, SKELETON_VERSION_MAJOR); /* version major */
+  write16le(op.packet+10, SKELETON_VERSION_MINOR); /* version minor */
+  write64le(op.packet+12, (ogg_int64_t)Int64_val(pres_num)); /* presentationtime numerator */
+  write64le(op.packet+20, (ogg_int64_t)Int64_val(pres_den)); /* presentationtime denominator */
+  write64le(op.packet+28, (ogg_int64_t)Int64_val(base_num)); /* basetime numerator */
+  write64le(op.packet+36, (ogg_int64_t)Int64_val(base_den)); /* basetime denominator */
+  /* both the numerator are zero hence handled by the memset */
+  write32le(op.packet+44, Int32_val(time)); /* UTC time, set to zero for now */
+
+  op.b_o_s = 1; /* its the first packet of the stream */
+  op.e_o_s = 0; /* its not the last packet of the stream */
+  op.bytes = 64; /* length of the packet in bytes */
+
+  packet = value_of_packet(&op);
+  free(op.packet);
+  CAMLreturn(packet);
+}
+
+/* Code from theorautils.c from ffmpeg2theora */
+CAMLprim value ocaml_ogg_skeleton_eos(value v)
+{
+  CAMLparam0();
+  ogg_packet op;
+
+  /* build the e_o_s packet */
+  memset (&op, 0, sizeof (op));
+  op.b_o_s = 0;
+  op.e_o_s = 1; /* its the e_o_s packet */
+  op.granulepos = 0;
+  op.bytes = 0; /* e_o_s packet is an empty packet */  
+
+  CAMLreturn(value_of_packet(&op));
+} 
+
